;This file is copyright (c) 1991, 1992 Informant Communications Group and the
;article author. The material here may be used in an application provided
;that this copyright/disclaimer information is kept in the original source
;file. The material presented here is provided "as is" and with no guarantee.
;Informant Communications Group/Paradox Informant assume no responsibility
;for the use or misuse of the material contained within.
;
;Contents        : procedures RptSetting,ChangeRptSet,GetCurrSet,
;                             DisplaySet,SetScr,ModifySet,SetErr
;
;Version         : 1.1
;
;Source File     : RPTSET.SC
;Author          : Jim Schwarz
;                  Paradox Informant
;
;Informant Issue : January 1992
;
;Description     : A series of procedures that allow for single step
;                  report settings through a single comprehensive display.
;
; Paradox Informant
; 8525 Elk Grove Blvd.
; Suite 126
; Elk Grove, CA  95624-1777
; Phone: (916) 686-6610
; Fax  : (916) 686-8497
; BBS  : (916) 686-4740
;-----------------------------------------------------------------------------

CREATELIB "RptSet"

;**************************************************;
; RptSetting() initializes global variable set and ;
; calls procedures based upon current mode.        ;
;**************************************************;
PROC RptSetting()
  PRIVATE Mode,ModeColor,MsgLn1,MsgLn2,
          Names,Pntr,Port,Set,DfltPntr

  Mode=SYSMODE()
  ModeColor=SYSCOLOR(5)
  ;
  ; DfltPntr is used as flag by GetCurrSet()
  ; for determination of default printer and
  ; for identifying same in LookUpPntr
  ;
  DfltPntr=0

  ARRAY Names[2]
  Names[1]=""  ;table name
  Names[2]=""  ;report name
  ;
  ; context sensitive messages displayed with
  ; the report settings screen
  ;
  ARRAY MsgLn1[8]
  ARRAY MsgLn2[8]
  MsgLn1[1]="Enter value for left margin.            "
  MsgLn2[1]="                                        "
  MsgLn1[2]="Enter value for page length.            "
  MsgLn2[2]="                                        "
  MsgLn1[3]="Enter value for page width.             "
  MsgLn2[3]="                                        "
  MsgLn1[4]="Press [F1] for list of available printer"
  MsgLn2[4]="options.                                "
  MsgLn1[5]="Press [F1] for list of available port   "
  MsgLn2[5]="options.                                "
  MsgLn1[6]="Enter or edit printer setup string.  Use"
  MsgLn2[6]="[ESC] to cancel changes.                "
  MsgLn1[7]="Enter starting page for output or leave "
  MsgLn2[7]="blank for full report.                  "
  MsgLn1[8]="Enter ending page for output or leave   "
  MsgLn2[8]="blank for full report.                  "
  ;
  ; current list of Paradox predefined
  ; printer setup strings
  ;
  ARRAY Pntr[13]
  Pntr[1] ="StandardPrinter"
  Pntr[2] ="Small-IBMgraphics"
  Pntr[3] ="Reg-IBMgraphics"
  Pntr[4] ="Small-Epson-MX/FX"
  Pntr[5] ="Small-Oki-92/93"
  Pntr[6] ="Small-Oki-82/83"
  Pntr[7] ="Small-Oki-192"
  Pntr[8] ="HPLaserJet"
  Pntr[9] ="HP-Portrait-66lines"
  Pntr[10]="HP-Landscape-Normal"
  Pntr[11]="HP-LandscpCompressed"
  Pntr[12]="Intl-IBMcompatible"
  Pntr[13]="Intl-IBMcondensed"
  ;
  ; current list of printer ports
  ;
  ARRAY Port[6]
  Port[1]="LPT1"
  Port[2]="LPT2"
  Port[3]="LPT3"
  Port[4]="COM1"
  Port[5]="COM2"
  Port[6]="AUX"
  ;
  ; array used to hold report settings data
  ;
  ARRAY Set[8]
  FOR i FROM 1 TO 8
    Set[i]=""
  ENDFOR

  IF Mode<>"Report"
    THEN DO_IT!
         GetRptName()
    ELSE ChangeRptSet()
  ENDIF
ENDPROC


;**************************************************;
; ChangeRptSet() manages modification of report    ;
; settings.  It calls GetCurrSet() to get current  ;
; report specifications for the given table/report ;
; name and changes settings with ModifySet()       ;
;**************************************************;
PROC ChangeRptSet()
  PRIVATE Item,Next,Pos

  Item=1
  Next=1
  Pos=1
  ;
  ; open data entry window
  ;
  SetScr()
  ;
  ; get and display current report settings
  ;
  GetCurrSet()

  WHILE Item > 0
    STYLE ATTRIBUTE 31
    ;
    ; display message for given field (Item)
    ;
    @15,19 ?? MsgLn1[Item]
    @16,19 ?? MsgLn2[Item]
    ;
    ; display Setup string overflow indicator
    ;
    @11,59
    IF LEN(Set[6])>24
      THEN ?? CHR(16)
      ELSE ?? " "
    ENDIF
    ;
    ; highlight current item
    ;
    STYLE ATTRIBUTE 113
    PAINTCANVAS
      ATTRIBUTE 113
      Item+5,34,Item+5,58
    ;
    ; if editing Setup string, call GetPntrStr()
    ;
    IF Item=6
      THEN Item=GetPntrStr()
           Next=Item
           LOOP
    ENDIF
    ;
    ; set cursor position to end-of-line
    ;
    Pos=LEN(Set[Item])+1
    @Item+5,Pos+33
    CANVAS ON
    CURSOR NORMAL

    WHILE True
      c=GETCHAR()
      ;
      ; printer and port types can only be selected
      ; from the available lookup tables
      ;
      IF Item=4 OR Item=5
        THEN IF c=8 OR (c>30 AND c<127)
               THEN BEEP
                    MESSAGE "Use [F1] to change or "+
                            "[Ctrl][Backspace] to clear..."
                    LOOP
             ENDIF
      ENDIF
      SWITCH
        CASE c>31 AND c<127 AND Pos<26: ;printable char
          Pos=Pos+1
          ?? CHR(c)
          Set[Item]=Set[Item]+CHR(c)
          LOOP
        CASE c=8 AND Pos>1:
          Pos=Pos-1
          @Item+5,Pos+33 ??" "
          @Item+5,Pos+33
          IF Pos=1
            THEN Set[Item]=""
            ELSE Set[Item]=SUBSTR(Set[Item],1,Pos-1)
          ENDIF
          LOOP
        CASE c=127:                    ;Ctrl Backspace
          Pos=1
          @Item+5,Pos+33 ??SPACES(24)
          @Item+5,Pos+33
          Set[Item]=""
          LOOP
        CASE c=-72:                    ;Up
          IF Item=1
            THEN BEEP
                 LOOP
            ELSE Next=Item-1
          ENDIF
          QUITLOOP
        CASE c=13 OR c=-80:            ;Enter or Down
          ;
          ; no range specification available
          ; when in Report Generator
          ;
          IF Item=8 OR (Item=6 AND Mode="Report")
            THEN BEEP
                 LOOP
            ELSE Next=Item+1
          ENDIF
          QUITLOOP
        CASE c=-59 AND Item=4:         ;F1
          ;
          ; get printer choice; if return value
          ; is greater than zero, reset
          ;
          Choice=LookUpPntr()
          IF Choice>0
            THEN Set[Item]=Pntr[Choice]
          ENDIF
          ;
          ; redraw screen without Printer list
          ;
          CANVAS OFF
          ECHO NORMAL
          ECHO OFF
          IF Mode<>"Report"
            THEN RptNameScr()
          ENDIF
          SetScr()
          DisplaySet()
          QUITLOOP
        CASE c=-59 AND Item=5:         ;F1
          ;
          ; get port choice; if return value is
          ; greater than zero, reset
          ;
          Choice=LookUpPort()
          IF Choice>0
            THEN Set[Item]=Port[Choice]
          ENDIF
          ;
          ; redraw screen without Port list
          ;
          CANVAS OFF
          ECHO NORMAL
          ECHO OFF
          IF Mode<>"Report"
            THEN RptNameScr()
          ENDIF
          SetScr()
          DisplaySet()
          QUITLOOP
        CASE c=-60:                    ;F2
          ;
          ; a report setting error will cause a move
          ; to the field of occurrence (Next>0);
          ; otherwise, settings will be saved and
          ; ChangeRptSet() exited (Next=0)
          ;
          Next=ModifySet()
          QUITLOOP
        CASE c=-65:                    ;F7
          ;
          ; a report setting error will cause a move
          ; to the field of occurrence without
          ; printing the report (Next<>0); otherwise,
          ; print report and return
          ;
          Next=ModifySet()
          IF Next<>0
            THEN QUITLOOP
          ENDIF

          MESSAGE "Sending report to printer..."
          IF Mode="Report"
            THEN MENU {OutPut} {Printer}
            ;
            ; if not Report mode, check for Range
            ;
            ELSE IF ISBLANK(Set[7])
                   THEN MENU {Report} {OutPut}
                        SELECT Names[1]
                        SELECT Names[2]
                        {Printer}
                   ELSE MENU {Report} {RangeOutPut}
                        SELECT Names[1]
                        SELECT Names[2]
                        {Printer}
                        SELECT NUMVAL(Set[7])
                        ;
                        ; if Range End is blank,
                        ; print entire report
                        ;
                        IF ISBLANK(Set[8])
                          THEN SELECT BLANKNUM()
                          ELSE SELECT NUMVAL(Set[8])
                        ENDIF
                 ENDIF
                 MENU
          ENDIF
          RETURN
        CASE c=27:                     ;Esc
          ;
          ; return without saving changes
          ;
          IF Mode<>"Report"
            THEN MENU
          ENDIF
          RETURN
        OTHERWISE:
          BEEP
      ENDSWITCH
    ENDWHILE
    CURSOR OFF
    CANVAS OFF
    ;
    ; remove highlight
    ;
    PAINTCANVAS
      ATTRIBUTE 31
      Item+5,34,Item+5,58
    Item=Next
  ENDWHILE
ENDPROC


;**************************************************;
; GetCurrSet() gets current report settings by     ;
; using MENUCHOICE() and places the data in Set[]. ;
;**************************************************;
PROC GetCurrSet()
  MENU
  IF Mode<>"Report"
    THEN {Report} {Change}
         SELECT Names[1] SELECT Names[2]
         ENTER
  ENDIF
  {Setting} {Margin}
  Set[1]=MENUCHOICE()

  ESC
  {PageLayout} {Length}
  Set[2]=MENUCHOICE()

  ESC
  {Width}
  Set[3]=MENUCHOICE()

  ESC ESC
  {Setup} {Custom}
  Set[5]=MENUCHOICE()

  ENTER
  Set[6]=MENUCHOICE()

  MENU
  ;
  ; find default printer if it has not yet
  ; been identified and proceed to display
  ; current settings
  ;
  IF DfltPntr=0
    THEN {Setting} {Setup} {Predefined}
         FOR i FROM 1 TO 14
           IF MATCH(MENUCHOICE(),"..*")
             THEN QUITLOOP
             ELSE RIGHT
           ENDIF
         ENDFOR
         Pntr[i]=Pntr[i]+"*"
         DfltPntr=i
         MENU
  ENDIF
  IF Mode<>"Report"
    THEN {Cancel} {Yes}
    ELSE ESC
  ENDIF
  ;
  ; reset Printer and Range vars
  ;
  Set[4]=""
  Set[7]=""
  Set[8]=""

  DisplaySet()
ENDPROC


;***************************************************;
; DisplaySet() displays current data (Set[]) in the ;
; the SetScr() window.                              ;
;***************************************************;
PROC DisplaySet()
  PRIVATE i

  CANVAS OFF
  STYLE ATTRIBUTE 31

  FOR i FROM 1 TO 8
    IF i=6
      THEN @i+5,34 ??FORMAT("w24",Set[i])
           LOOP
    ENDIF
    @i+5,34 ??Set[i]
  ENDFOR

  STYLE ATTRIBUTE 113
ENDPROC


;**************************************************;
; SetScr() opens a window on the canvas for report ;
; setting data entry/edit.  Since a call is made   ;
; to get the current settings whenever the window  :
; is opened, this fact is MESSAGEed here before    ;
; the canvas is turned off.                        ;
;**************************************************;
PROC SetScr()
  MESSAGE "Getting current Report settings..."
  CANVAS OFF
  STYLE
  @0,0 ?? " [F2]   Save Settings   "+
          " [F7]  Print report"+spaces(36)
  @1,0 ?? " [ESC]  Cancel Changes"+spaces(57)
  STYLE ATTRIBUTE ModeColor
  @0,68 ??"RptSetting"
  ;
  ; paint shadow
  ;
  PAINTCANVAS
    ATTRIBUTE 8
    6,19,18,61

  STYLE ATTRIBUTE 31
  @5,18
  SETMARGIN 18

TEXT
MODIFY SETTINGSĿ
       Margin:                           
       Length:                           
        Width:                           
      Printer:                           
 Printer Port:                           
 Setup string:                           
  Range start:                           
    Range end:                           
Ĵ
                                         
                                         

ENDTEXT

  SETMARGIN 0

ENDPROC


;***************************************************;
; ModifySet() modifies the report settings with the ;
; provided data (Set[]).  ErrorProc is used to halt ;
; the modification process whenever an invalid      ;
; report specifiction is encountered.  The field    ;
; number in which the error occurred is returned to ;
; ChangeRptSet().  A zero is returned if no error.  ;
;***************************************************;
PROC ModifySet()
  PRIVATE ValidSet,ErrorProc

  ValidSet=True
  ErrorProc="SetErr"

  MESSAGE "Changing Report settings..."

  MENU
  IF Mode<>"Report"
    THEN {Report} {Change}
         SELECT Names[1] SELECT Names[2]
         ENTER
         MENU
  ENDIF
  {Setting} {Margin}
  SELECT Set[1]
  IF NOT ValidSet
    THEN MENU
         IF Mode="Report"
           THEN ESC
           ELSE {Cancel} {Yes}
         ENDIF
         RETURN 1
  ENDIF

  MENU
  {Setting} {PageLayout} {Length}
  SELECT Set[2]
  IF NOT ValidSet
    THEN MENU
         IF Mode="Report"
           THEN ESC
           ELSE {Cancel} {Yes}
         ENDIF
         RETURN 2
  ENDIF

  MENU
  {Setting} {PageLayout} {Width}
  SELECT Set[3]
  IF NOT ValidSet
    THEN MENU
         IF Mode="Report"
           THEN ESC
           ELSE {Cancel} {Yes}
         ENDIF
         RETURN 3
  ENDIF

  MENU
  IF NOT ISBLANK(Set[4])
    THEN {Setting} {Setup} {Predefined}
         SELECT Set[4]
         MENU
  ENDIF
  {Setting} {Setup} {Custom}
  SELECT Set[5]
  ENTER

  MENU
  IF ISBLANK(Set[4])
    THEN {Setting} {Setup} {Custom}
         ENTER
         SELECT Set[6]
         MENU
  ENDIF

  IF Mode<>"Report"
    THEN DO_IT!
    ELSE ESC
  ENDIF
  RETURN 0
ENDPROC


;**********************************************;
; SetErr() is the procedure called whenever an ;
; error is encountered by ModifySet().  The    ;
; Paradox error message is displayed and the   ;
; user is ultimately placed into that field to ;
; make appropriate changes.                    ;
;**********************************************;
PROC SetErr()
  PRIVATE ErrorProc

  BEEP
  MESSAGE ERRORMESSAGE()+"..."
  SLEEP 2000
  ValidSet=False
  RETURN 1
ENDPROC


;*************************************************;
; GetPntrStr() manages the viewing and editing of ;
; a printer setup string longer than the display  ;
; area by allowing the string to be scrolled      ;
; horizontally.                                   ;
;*************************************************;
PROC GetPntrStr()
  PRIVATE c,Pos,First,Last,MaxLen,NextItem,
          TempStr,TempFirst,TempLast,
          TempPos,TempStrLen
  ;
  ; set maximum string length
  ;
  MaxLen   = 100
  ;
  ; NextItem is returned to the calling
  ; procedure for field depart direction
  ;
  NextItem = 0

  Last   = LEN(Set[6])
  StrLen = Last
  Pos    = Last+1
  First  = 1
  IF Last>24
    THEN First = Last-23
         Pos   = 25
  ENDIF
  ;
  ; temporary variables used to undo changes
  ;
  TempStr    = Set[6]
  TempFirst  = First
  TempLast   = Last
  TempPos    = Pos
  TempStrLen = StrLen

  CURSOR NORMAL

  WHILE NextItem=0
    ;
    ; display field length indicators; left
    ; and right arrow heads indicating text
    ; beyond that currently in view
    ;
    STYLE ATTRIBUTE 31
    @ 11,33
    IF First>1
      THEN ?? CHR(17)
      ELSE ?? " "
    ENDIF
    @ 11,59
    IF Last<StrLen
      THEN ?? CHR(16)
      ELSE ?? " "
    ENDIF
    ;
    ; highlight current field, display
    ; last 24 characters of string and
    ; leave cursor at end
    ;
    STYLE ATTRIBUTE 113
    @11,34 ?? SUBSTR(Set[6],First,24)
    CANVAS ON

    WHILE True
      @11,33+Pos
      ;
      ; get user input
      ;
      c=GETCHAR()
      SWITCH
        CASE c>31 AND c<127:                ;printable chr
          IF StrLen=MaxLen
            THEN BEEP
                 LOOP
          ENDIF
          Len1=First+Pos-2
          Len2=StrLen-Len1-1
          Set[6]=SUBSTR(Set[6],1,Len1)+
                 CHR(c)+
                 SUBSTR(Set[6],Len1+2,Len2)
          IF Pos>(StrLen-First+1)
            THEN StrLen=StrLen+1
                 Last=Last+1
          ENDIF
          IF Pos=25
            THEN First=First+1
                 CANVAS OFF
                 QUITLOOP
            ELSE ?? CHR(c)
                 Pos=Pos+1
          ENDIF
          LOOP
        CASE c=8 AND Pos>(Last-First+1):    ;Backspace
          IF Pos>1
            THEN Pos=Pos-1
            ELSE IF First>1
                   THEN First=First-1
                   ELSE BEEP
                        LOOP
                 ENDIF
          ENDIF
          Last=Last-1
          StrLen=StrLen-1
          Set[6]=SUBSTR(Set[6],1,StrLen)
          @11,33+Pos ??" "
          LOOP
        CASE c=-83 AND Pos<=(Last-First+1): ;Del
          StrLen=StrLen-1
          Len1=First+Pos-2
          Len2=StrLen-Len1
          Set[6]=SUBSTR(Set[6],1,Len1)+
                 SUBSTR(Set[6],Len1+2,Len2)
          IF StrLen<Last
            THEN Last=StrLen
          ENDIF
          CANVAS OFF
          @11,34 ?? SPACES(24)
          QUITLOOP
        CASE c=-75:                         ;Left
          IF Pos>1
            THEN Pos=Pos-1
            ELSE IF First>1
                   THEN First=First-1
                        IF (Last-First)>23
                          THEN Last=Last-1
                        ENDIF
                        CANVAS OFF
                        QUITLOOP
                   ELSE BEEP
                 ENDIF
          ENDIF
          LOOP
        CASE c=-77:                         ;Right
          IF Pos<=(Last-First+1)
            THEN Pos=Pos+1
            ELSE IF Last<StrLen
                   THEN First=First+1
                        Last=Last+1
                        CANVAS OFF
                        @11,34 ?? SPACES(24)
                        QUITLOOP
                   ELSE BEEP
                 ENDIF
          ENDIF
          LOOP
        CASE c=-72:                         ;Up
          NextItem=5
          QUITLOOP
        CASE c=13 OR c=-80:                 ;Enter or Down
          ;
          ; cannot use Range output if
          ; in Report generator
          ;
          IF Mode="Report"
            THEN BEEP
                 LOOP
            ELSE NextItem=7
                 QUITLOOP
          ENDIF
        CASE c=27:                          ;Esc
          ;
          ; exit if no change; otherwise,
          ; undo changes and loop
          ;
          IF Set[6]=TempStr
            THEN IF Mode<>"Report"
                   THEN MENU
                 ENDIF
                 RETURN 0
            ELSE Set[6] = TempStr
                 First  = TempFirst
                 Last   = TempLast
                 StrLen = TempStrLen
                 Pos    = TempPos
                 CANVAS OFF
                 @11,34 ?? SPACES(24)
                 QUITLOOP
          ENDIF
        OTHERWISE:
          BEEP
      ENDSWITCH
    ENDWHILE
  ENDWHILE
  CANVAS OFF
  STYLE ATTRIBUTE 31
  ;
  ; display from beginning of string and
  ; return field departure direction
  ;
  @11,33 ?? " "+FORMAT("w24",Set[6])+" "
  RETURN NextItem
ENDPROC


WRITELIB "RptSet" RptSetting,ChangeRptSet,GetCurrSet,
                  DisplaySet,SetScr,ModifySet,SetErr,
                  GetPntrStr
RELEASE PROCS RptSetting,ChangeRptSet,GetCurrSet,
              DisplaySet,SetScr,ModifySet,SetErr,
              GetPntrStr

